"""Standard Django model definitions"""
from django.db import models


# Create your models here.
class MediumType(models.Model):
    """Enum for media types"""
    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=255)
    description = models.CharField(max_length=255)

    def __str__(self):
        return self.description

    class Meta:
        """Definition of MySQL Table"""
        db_table = "mediumType"


class Medium(models.Model):
    """Model for media classes"""
    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=255)
    medium_type = models.ForeignKey(MediumType, on_delete=models.CASCADE)

    def __str__(self):
        return self.name

    class Meta:
        """Definition of MySQL Table"""
        db_table = "medium"


class Tag(models.Model):
    """Model for created topic labels for articles"""
    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=255)
    description = models.CharField(max_length=255)

    class Meta:
        """Definition of MySQL Table"""
        db_table = "tag"


class Article(models.Model):
    """Model of a single article, which is connected to a medium an several tags"""
    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=255)
    medium = models.ForeignKey(Medium, on_delete=models.CASCADE)
    author = models.CharField(max_length=255)
    date_of_publication = models.DateField('date published')
    reach = models.IntegerField(default=0)
    tags = models.ManyToManyField(Tag, through='ArticleTag', through_fields=('article', 'tag',))

    def __str__(self):
        return self.name

    class Meta:
        """Definition of MySQL Table"""
        db_table = "article"


class ArticleTag(models.Model):
    """Model for many-to-many relation between article and tag"""
    article = models.ForeignKey(Article, on_delete=models.CASCADE)
    tag = models.ForeignKey(Tag, on_delete=models.CASCADE)

    class Meta:
        """Definition of MySQL Table"""
        db_table = "articleTag"


class Timeline(models.Model):
    """Model for accumulated article data (is a MySQL View) summarized by year and month"""
    name = models.CharField(max_length=255)
    year = models.IntegerField(default=1990)
    month = models.IntegerField(default=1)
    article_count = models.IntegerField(default=0)
    reach = models.DecimalField(max_digits=200, decimal_places=0, default=0)

    def __str__(self):
        return self.name + ", " + \
               str(self.year) + ", " + \
               str(self.month) + ", " + \
               str(self.article_count) + ", " + \
               str(self.reach)

    class Meta:
        """Definition of MySQL Table"""
        managed = False
        db_table = "tag_reach"


class Overview(models.Model):
    """Model for compounded objects"""
    name = models.CharField(max_length=255)
    article_name = models.CharField(max_length=255)
    medium_name = models.CharField(max_length=255)
    date_of_publication = models.DateField('date published')
    reach = models.DecimalField(max_digits=200, decimal_places=0, default=0)

    class Meta:
        """Definition of MySQL Table"""
        managed = False
        db_table = "tag_article"
