"""Standard django REST Framework serializer module"""

import logging
from rest_framework import serializers
from articles.models import Article, Medium, Timeline, MediumType, Tag, Overview

# Get an instance of a logger
logger = logging.getLogger(__name__)


class MediumTypeSerializer(serializers.ModelSerializer):
    """ Serializer fpr MediumType """

    class Meta:
        """ Definition of model and used fields """
        model = MediumType
        fields = '__all__'

    def create(self, validated_data):
        """ Creates or updates a given mediumType """
        medium_type, created = MediumType.objects.update_or_create(name=validated_data.get('name', None),
                                                                   defaults={
                                                                       'description': validated_data.get('description',
                                                                                                         None)})
        if created:
            logger.debug("created new medium_type: %s", validated_data.get('name', None))

        return medium_type


class MediumSerializer(serializers.ModelSerializer):
    """ Serializer for media """

    class Meta:
        """ Definition of model and used fields """
        model = Medium
        fields = '__all__'

    # if the name of the print is already in database it just updates the medium type
    def create(self, validated_data):
        """ creates or updates a given medium with its medium_type """
        medium, created = Medium.objects.update_or_create(name=validated_data.get('name', None),
                                                          defaults={
                                                              'medium_type': validated_data.get('medium_type', None)})
        if created:
            logger.debug("created new medium: %s", validated_data.get('name', None))

        return medium


class ArticleSerializer(serializers.ModelSerializer):
    """ Serializer for articles """
    medium = MediumSerializer(many=False)

    class Meta:
        """ Definition of model and used fields """
        model = Article
        fields = '__all__'

    # create or update
    def create(self, validated_data):
        """ creates or updates a new article with its medium and tags """
        # cut out medium data because we have to compute it differently
        medium_data = validated_data.pop('medium')
        # I don't know why I init a new serializer but the one from the top is not accessible
        medium_ser = MediumSerializer(many=False)
        medium_obj = medium_ser.create(medium_data)
        # ** gives you kwargs, gives all arguments as dictionary
        article, created = Article.objects.update_or_create(medium=medium_obj, **validated_data)

        if created:
            logger.debug("created new article: %s", validated_data.get('name', None))
        return article


class RawArticleSerializer(serializers.Serializer):
    """ Serializer for a json representation of an article """

    def update(self, instance, validated_data):
        """ Not implemented: Don't need to update """

    def create(self, validated_data):
        """ Not implemented: Don't need to create """

    name = serializers.CharField(max_length=200)
    medium_name = serializers.CharField(max_length=100)
    medium_type = serializers.CharField(max_length=100)
    author = serializers.CharField(max_length=100)
    reach = serializers.IntegerField()
    tags = serializers.ListField(
        child=serializers.CharField(max_length=100)
    )
    date_of_publication = serializers.DateField()

    @staticmethod
    def convert_to_db(validated_data):
        """ Converts the json into the database pattern """
        medium_type, mt_created = MediumType.objects.update_or_create(name=validated_data.get('medium_type'))
        if mt_created:
            logger.debug("created new medium_type: %s", validated_data.get('medium_type', None))
        medium, medium_created = Medium.objects.update_or_create(name=validated_data.get('medium_name'),
                                                                 medium_type=medium_type)
        if medium_created:
            logger.debug("created new medium: %s", validated_data.get('medium_name', None))
        article, article_created = Article.objects.update_or_create(name=validated_data.get('name'),
                                                                    medium=medium,
                                                                    author=validated_data.get('author'),
                                                                    reach=validated_data.get('reach'),
                                                                    date_of_publication=validated_data.get(
                                                                        'date_of_publication'))
        if article_created:
            logger.debug("created new article: %s", validated_data.get('name', None))

        for tag_name in validated_data.get('tags'):
            tag, tag_created = Tag.objects.update_or_create(name=tag_name)
            if tag_created:
                logger.debug("created new tag: %s", tag_name)
            article.tags.add(tag)
        article.save()


class TimelineSerializer(serializers.ModelSerializer):
    """ Serializer for timeline """
    class Meta:
        """ Definition of model and used fields """
        model = Timeline
        fields = ['name', 'year', 'month', 'article_count', 'reach']


class OverviewSerializer(serializers.ModelSerializer):
    """ Serializer for overview """
    class Meta:
        """ Definition of model and used fields """
        model = Overview
        fields = ['name', 'article_name', 'medium_name', 'date_of_publication', 'reach']
