""" Test class for article module """

import json

from datetime import date
from django.test import TestCase, TransactionTestCase
from rest_framework.test import APIRequestFactory

from articles.models import Article, MediumType, Medium
from articles.views import ArticleRaw


class MediumTypeTest(TransactionTestCase):
    """ database test medium_type """
    reset_sequences = True

    def setUp(self):
        MediumType.objects.create(name='local', description='Lokale Zeitung')

    def test_db_call(self):
        """ Tests get medium_type """
        local_print = MediumType.objects.get(name='local')
        self.assertEqual(local_print.description, 'Lokale Zeitung')
        self.assertEqual(local_print.id, 1)


class MediumTest(TransactionTestCase):
    """ database test medium """
    reset_sequences = True

    def setUp(self):
        MediumType.objects.create(name='new local', description='Lokale Zeitung')
        Medium.objects.create(name='Meine Zeitung', medium_type_id=1)

    def test_db_call(self):
        """ Tests get media """
        my_print = Medium.objects.get(name='Meine Zeitung')
        self.assertEqual(my_print.name, 'Meine Zeitung')
        self.assertEqual(my_print.medium_type.name, 'new local')
        self.assertEqual(my_print.medium_type.id, 1)


class ArticleTest(TransactionTestCase):
    """ database test medium """
    reset_sequences = True

    def setUp(self):
        MediumType.objects.create(name='my local', description='Lokale Zeitung')
        Medium.objects.create(name='JDs Zeitung', medium_type_id=1)
        Article.objects.create(name='Dr. Acula', author='JD', reach=12, medium_id=1, date_of_publication=date.today())

    def test_author(self):
        """ Tests get medium_type """
        article = Article.objects.get(name='Dr. Acula')
        self.assertEqual(article.author, 'JD')
        self.assertEqual(article.medium.name, 'JDs Zeitung')


class RawArticleAPI(TestCase):
    """ test raw article serializer """
    def setUp(self):
        self.factory = APIRequestFactory()
        self.json_article = {"name": "Was Corona mit unserer Gesellschaft macht ", "author": "None", "reach": 69556,
                             "tags": ["UP"], "medium_name": "Lausitzer Rundschau", "medium_type": "local",
                             "date_of_publication": "2020-05-30"}
        self.json_article_wrong = {"name": "Was Corona mit unserer Gesellschaft macht ", "author": "None",
                                   "reach": 69556, "tags": ["UP"], "medium_namee": "Lausitzer Rundschau",
                                   "medium_type": "local", "date_of_publication": "2020-05-30"}

    def test_post_raw_article(self):
        """ tests a correct post message """
        request = self.factory.post('/articles/raws', json.dumps(self.json_article), content_type='application/json')
        response = ArticleRaw.as_view()(request)
        self.assertEqual(response.status_code, 201)

    def test_wrong_post_raw_article(self):
        """ tests a wrong post message """
        request = self.factory.post('/articles/raws', json.dumps(self.json_article_wrong),
                                    content_type='application/json')
        response = ArticleRaw.as_view()(request)
        self.assertEqual(response.status_code, 400)
