""" Views with OpenAPI annotations """

from django.db import transaction
from django.utils.decorators import method_decorator
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework import generics, status
from drf_yasg.utils import swagger_auto_schema
from articles.models import Article, Timeline, Medium, MediumType, Overview
from articles.serializers import ArticleSerializer, TimelineSerializer, MediumSerializer, MediumTypeSerializer, \
    RawArticleSerializer, OverviewSerializer


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="gets a list of all articles"
))
@method_decorator(name='post', decorator=swagger_auto_schema(
    operation_description="inserts an new article or updates if the name is already in the db"
))
class ArticleList(generics.ListCreateAPIView):
    """ Lists all articles """
    queryset = Article.objects.all()
    serializer_class = ArticleSerializer


@method_decorator(name='post', decorator=swagger_auto_schema(
    operation_description="post a oneliner",
    request_body=RawArticleSerializer
))
class ArticleRaw(APIView):
    """ Post a raw article to be converted for database """

    @transaction.atomic
    def post(self, request):
        """ transactional post method """
        raw_serializer = RawArticleSerializer(data=request.data)
        if raw_serializer.is_valid():
            raw_serializer.convert_to_db(request.data)
            return Response(request.data, status=status.HTTP_201_CREATED)
        return Response(raw_serializer.errors, status=status.HTTP_400_BAD_REQUEST)


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="shows all details about one article selected by Id"
))
class ArticleDetail(generics.RetrieveUpdateDestroyAPIView):  # pylint: disable=too-many-ancestors
    """ Get details of article """
    queryset = Article.objects.all()
    serializer_class = ArticleSerializer


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="gets a list of all medias"
))
@method_decorator(name='post', decorator=swagger_auto_schema(
    operation_description="inserts an new medium or updates if the name is already in the db"
))
class MediumList(generics.ListCreateAPIView):
    """ List all media """
    queryset = Medium.objects.all()
    serializer_class = MediumSerializer


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="shows all details about one medium selected by Id"
))
class MediumDetail(generics.RetrieveUpdateDestroyAPIView):  # pylint: disable=too-many-ancestors
    """ Gets details of medium """
    queryset = Medium.objects.all()
    serializer_class = MediumSerializer


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="gets a list of all types of media"
))
@method_decorator(name='post', decorator=swagger_auto_schema(
    operation_description="inserts an new medium_type or updates if the name is already in the db"
))
class MediumTypeList(generics.ListCreateAPIView):
    """ List all medium_types """
    queryset = MediumType.objects.all()
    serializer_class = MediumTypeSerializer


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="shows all details about one medium_type selected by Id"
))
class MediumTypeDetail(generics.RetrieveUpdateDestroyAPIView):  # pylint: disable=too-many-ancestors
    """ Get details of medium_type """
    queryset = MediumType.objects.all()
    serializer_class = MediumTypeSerializer


class TimelineList(generics.ListAPIView):
    """ List all timeline objects """
    queryset = Timeline.objects.all()
    serializer_class = TimelineSerializer


class OverviewList(generics.ListAPIView):
    """ List all overview objects """
    queryset = Overview.objects.all()
    serializer_class = OverviewSerializer
